/* ========================================================================
 *                                ERNDLCON.H
 * ------------------------------------------------------------------------
 *  Instantiations of the LinConRandGen class.
 *
 *  Version 0.9
 *
 *  Copyright 1996-1997 by G. Wade Johnson
 *   Use of this code is placed in the public domain as long as the
 *   copyright notice is retained.
 */

#if !defined( ERNDLCON_H )
  #define ERNDLCON_H

  #if !defined( RANDLCON_H )
    #include "randlcon.h"
  #endif

/* =========================================================================
 *  Random number generator based on Borland's RunTime Library.
 *  Chi-squared test on seeds from 1 to 32767
 *     Success: 27671
 *     Failure: 5095
 */
  class  BorlandRandGen : public LinConRandGen
   {
  public:
    EXPLICIT BorlandRandGen( size_rand seed=1 ) :
      LinConRandGen( 0x015a4e35L, 1L, (seed & 0x7fff) )
      { }
    virtual  void       IntSeed( size_rand seed )
     {
      LinConRandGen::IntSeed( (seed & 0x7fff) );
     }
    virtual size_rand   Number( void )
     {
      return ((unsigned)(LinConRandGen::Number() >> 16) & 0x7fff);
     }
    virtual  size_rand   MaxRandom( void ) const { return 0x7fff; }
    virtual  DIFFRET(RandomGenerator,BorlandRandGen)  *ReCreate( void ) const
     {
      return new BorlandRandGen( Seed() );
     }
   } ;
   




/* =========================================================================
 *  Random number generator based on example from _The Annotated ANSI C
 *  Standard_.
 */
  class  StdCRandGen : public LinConRandGen
   {
  public:
    EXPLICIT StdCRandGen( size_rand seed=1 ) :
      LinConRandGen( 1103515245L, 12345L, (seed&0x7fff) )
      { }
    virtual  void       IntSeed( size_rand seed )
     {
      LinConRandGen::IntSeed( (seed & 0x7fff) );
     }
    virtual size_rand   Number( void )
     {
      //  The example uses ((Number/65536)%32768). 
      return ((unsigned)(LinConRandGen::Number() >> 16) & 0x7fff);
     }
    virtual  size_rand   MaxRandom( void ) const { return 0x7fff; }
    virtual  DIFFRET(RandomGenerator,StdCRandGen)  *ReCreate( void ) const
     {
      return new StdCRandGen( Seed() );
     }
   } ;
   

#endif
