#!perl
# ========================================================================
#                               MAKERNG.PL
# ------------------------------------------------------------------------
#  Create template files for a new random number generator class.
#
#  Version 0.2
#
#  Copyright 1996-1997 by G. Wade Johnson
#   Use of this code is placed in the public domain as long as the
#   copyright notice is retained.
#

%Fields =
  (  'FileName',    'RandJunk',
     'Generator',   'JunkRandGen',
     'Description', 'junky generator',
     'Parent',      'RandomGenerator',
     'Type',        'Concrete',
     'TokenName',   ''
  ) ;
@FieldNames = ( '', 'FileName', 'Generator', 'Description', 'Parent', 'Type',
                'TokenName' );

@GeneratorTypes = ( '', 'Abstract', 'Concrete', 'Function' );

$HeaderStart =<<'EOF';
/* =========================================================================
 *                                 $$FileName$$.H
 * -------------------------------------------------------------------------
 *  Interface file for $$Description$$ as an object.
 */

#if !defined( $$FileNameUC$$_H )
  #define $$FileNameUC$$_H

  #if !defined( RANDGEN_H )
    #include "randgen.h"
  #endif
EOF
$StartGenClass=<<'EOF';
  // Insert parent header file here


  class  $$Generator$$ : public $$Parent$$
   {
  private:
    // add private data

  private:
    // protect the = operator and copy constructor so they cannot be used.
    $$Generator$$& operator=( $$Generator$$ const & );
    $$Generator$$( $$Generator$$ const & );

  protected:
    // access methods for private data.

  public:
    $$Generator$$( /* fill in arguments */ );
    virtual   ~$$Generator$$();

EOF

$TokenDecl=<<'EOF';
    virtual RandGenToken  CreateToken( void ) const;
    virtual int     SetToken( const RandGenToken &rToken );
EOF

$ConcreteCode=<<'EOF';
    virtual DIFFRET($$Generator$$)  *ReCreate( void ) const
     {
      return new $$Generator$$( /* fill in arguments */ );
     }

$$TokenDecl$$

    virtual void    Reset( void );

    virtual  void   IntSeed( size_rand seed );

    virtual  size_rand   Number( void );
EOF

$HeaderEnd =<<'EOF';
   } ;

#endif
EOF

$FunctionHeaderCode=<<'EOF';
  class $$Generator$$
   {
  private:
    RandomGenerator  *myRNG;

  private:
    // protect the = operator and copy constructor so they cannot be used.
    $$Generator$$( $$Generator$$ const &rhs );
    $$Generator$$& operator=( $$Generator$$ const &rhs );

  public:
    $$Generator$$( /* fill in arguments */ );
    ~$$Generator$$( void );

    size_rand   operator()( void );
EOF

$TokenDef=<<'EOF';
class  $$TokenName$$ : public RandGenTokenImpl
 {
  private:
    // protect the = operator and copy constructor so they cannot be used.
    $$TokenName$$ &operator=( const $$TokenName$$ & /* rhs */ );
    $$TokenName$$( const $$TokenName$$ & );

public:
  virtual   ~$$TokenName$$( void ) { }

  virtual _STRING   CreateInitString( void ) const;
  virtual int   Initialize( istream &InitStream );
  virtual $$TokenName$$   *Clone() const
   {
    return new $$TokenName$$( /* fill in arguments */ );
   }

  void       Print( ostream &os ) const
   {
    // write the token to os
   }
  void       Read( istream &is )
   {
    // read the token from is
   } 

private:
  friend class $$Generator$$;
  $$TokenName$$( /* place constructor arguments here */ )
   { }

  // Place private data here
 } ;





_STRING    $$TokenName$$::CreateInitString( void ) const
 {
  ostrstream  aString;

  // write your init string to String.

  return _STRING( aString.str() );
 }




int   $$TokenName$$::Initialize( istream &InitStream )
 {
  // Initialize the token from InitStream

  return 1;
 }



RandGenToken    $$Generator$$::CreateToken( void ) const
 {
  return  $$TokenName$$( /* place constructor arguments here */ );
 }



int   $$Generator$$::SetToken( const RandGenToken &rtok )
 {
  const $$TokenName$$  *pToken =
                   DYNACAST( const $$TokenName$$ * )(rtok.Implementation());

  if(pToken)
   {
    // Reset my data from the token

    return 1;
   }

  return 0;
 }



EOF

$Source=<<'EOF';
/* =========================================================================
 *                                $$FileName$$.CPP
 * -------------------------------------------------------------------------
 *  Implementation file for $$Description$$ as an object.
 */

#include "environ.h"
#include "$$FileName$$.h"

$$TokenDef$$

$$Generator$$::$$Generator$$( /* fill in arguments */ )
 : $$Parent$$( /* fill in arguments */ )
 {
  //  Here's where the fun begins
 }



$$Generator$$::~$$Generator$$( void )
 {
  //  Clean up after yourself.
 }


//  This is where the rest of the functionality is defined.
EOF
$ConcreteSource=<<'EOF';


size_rand   $$Generator$$::Number( void )
 {
  //  generate your next number here
 }



EOF

#
#  This is the actual code.
#
for(;;)
 {
  print "\n"x25;
  for($i=1;$i < scalar(@FieldNames);$i++)
   {
    print " $i\t$FieldNames[$i]\t$Fields{$FieldNames[$i]}\n";
   }

  print "\nSelect option to change or 0 to generate: ";
  chop($input = <>);

  exit 0 if("" eq $input);

  $select = $input + 0;
  last if(0 == $select);

  next if($select < 0 || $#FieldNames < $select);

  if(1 == $select)
   {
    do
     {
      print "Select a file name for this class's files: ";
      chop($input = <>);

     } while("" ne $input && &fileInUse( $input ));

    $Fields{$FieldNames[1]} = $input if "" ne $input;
   }
  elsif(2 == $select)
   {
    print "Select a name for this generator class: ";
    chop($input = <>);

    $Fields{$FieldNames[2]} = $input if "" ne $input;
   }
  elsif(3 == $select)
   {
    print "Select a description for this class: ";
    chop($input = <>);

    $Fields{$FieldNames[3]} = $input if "" ne $input;
   }
  elsif(4 == $select)
   {
    print "Select a name for this generator's parent class: ";
    chop($input = <>);

    $Fields{$FieldNames[4]} = $input if "" ne $input;
   }
  elsif(5 == $select)
   {
    &selectGeneratorType();
   }
  elsif(6 == $select)
   {
    print "Select a name for this generator's token class: ";
    chop($input = <>);

    $Fields{$FieldNames[6]} = $input;
   }
  else
   {
    print "Unrecognized selection.\n";
    $input = <>;
   }

 }

$Fields{'FileNameUC'} = "\U$Fields{'FileName'}";

if($Fields{'TokenName'})
 {
  $Fields{'TokenDef'}  = &substituteFields( $TokenDef );
  $Fields{'TokenDecl'} = &substituteFields( $TokenDecl );
 }

&makeGeneratorHeader();
&makeGeneratorSource();


sub  fileInUse
 {
  local( $file ) = @_[0];
  local( $input );

  return 0 unless -e "$file.h" && -e "$file.cpp";

  do
   {
    print "\'$file\' is in use. Overwrite? ";

    chop($input = <>);

   } while($input !~ /^[yn]/i);

  $input =~ /^n/i;
 }



sub  selectGeneratorType
 {
  local( $input, $select );

  print "\n"x25;
  for($i=1;$i < scalar(@GeneratorTypes);$i++)
   {
    print " $i\t$GeneratorTypes[$i]\n";
   }

  print "\nSelect a generator type: ";
  chop($input = <>);

  $select = $input + 0;

  $Fields{$FieldNames[5]} = $GeneratorTypes[$select]
                                 if(1 < $select && $select < $#GeneratorTypes);
 }


sub   makeGeneratorHeader
 {
  local( $FileName );
  $FileName = "$Fields{'FileName'}.h";

  open( OUT, "> $FileName" ) || die "Unable to open $FileName\n";
  print OUT &substituteFields( $HeaderStart );

  if('Function' ne $Fields{'Type'})
   {
    print OUT &substituteFields( $StartGenClass );
   }
  if('Concrete' eq $Fields{'Type'})
   {
    print OUT &substituteFields( $ConcreteCode );
   }
  elsif('Abstract' eq $Fields{'Type'})
   {
     # Nothing special we do here.
   }
  elsif('Function' eq $Fields{'Type'})
   {
    print OUT &substituteFields( $FunctionHeaderCode );
   }

  print OUT $HeaderEnd;
  close OUT;
 }



sub   makeGeneratorSource
 {
  local( $FileName );
  $FileName = "$Fields{'FileName'}.cpp";

  open( OUT, "> $FileName" ) || die "Unable to open $FileName\n";
  print OUT &substituteFields( $Source );
  print OUT &substituteFields( $ConcreteSource );

  close OUT;
 }


sub  substituteFields
 {
  local( $str ) = @_[0];

  $str =~ s/\$\$([^\s\$]+)\$\$/$Fields{"$1"}/g;

  $str;
 }
