/* ========================================================================
 *                                 RAND.CPP
 * ------------------------------------------------------------------------
 *  Program to test RNG class interface tests.
 *
 *  Version 0.9
 *
 *  Copyright 1995-1997 by G. Wade Johnson
 *   Use of this code is placed in the public domain as long as the
 *   copyright notice is retained.
 */


#include <stdlib.h>
#include "environ.h"

#include _IOSTREAM_HEADER
#include _STRSTREAM_HEADER
#include _IOMANIP_HEADER


#if defined( __BCPLUSPLUS__ )
  #include <conio.h>
  #include <alloc.h>
#endif

#include "rand_all.h"
#include "erndmcnm.h"
#include "erndsecm.h"
#include "erndlcon.h"
#include "erndmcon.h"
#include "erndlfib.h"


void    RNGInterfaceTest( char *pszLabel, RandomGenerator *pRNG );

void   enter( void ) { /* if(!getch()) getch(); */ } // used in debugging.

int    main( void )
 {
  RNGInterfaceTest( "Borland", new BorlandRandGen( 818UL ) );

  RNGInterfaceTest( "Linear Congruential",
                    new LinConRandGen( 54237L, 1UL, 923UL ) );

  RNGInterfaceTest( "Multiplicative Congruential",
                    new MultConRandGen( 54237L, 923UL )
                  );

  RNGInterfaceTest( "Inverse Congruential w/ mod",
                    new InvConModRandGen( 54236L, 923UL, 0xFFFFFFEFUL, 23UL ) );

  RNGInterfaceTest( "Inverse Congruential",
                    new InvConRandGen( 53237L, 915UL, 23UL ) );

  RNGInterfaceTest( "Shuffled M (Borland, Multiplicative)",
                    new ShuffledMRandGen(  53,
                               new BorlandRandGen( 1111UL ),
                               new MultConRandGen( 54237L, 927UL )
                              )
                  );

  RNGInterfaceTest( "Difference (Borland, Multiplicative)",
                    new DifferenceRandGen( new BorlandRandGen( 1111UL ),
                                           new MultConRandGen( 54237L, 927UL )
                                          )
                  );

  RNGInterfaceTest( "Byte Table-driven",
                    new DecimalTableRandGen( "rand.rnt" ) );

  RNGInterfaceTest( "Number Table-driven",
                    new NumberTableRandGen( "numrand.rn" ) );

  RNGInterfaceTest( "Second-order Congruential",
                    new SecConRandGen( -314159269L, 271828183L,
                    1UL, 923UL ) );

  RNGInterfaceTest( "Dwyer 1", new Dwyer1RandGen( 123UL ) );

  RNGInterfaceTest( "Linear Congruential with modulus",
                    new LinConModRandGen( 314159UL, 12345UL, 0x7fffffffUL,
                                          123UL )
                  );

  RNGInterfaceTest( "Knuth 28", new Knuth28RandGen( 123UL ) );

  RNGInterfaceTest( "Constant", new ConstantRandGen( 57UL ) );

  RNGInterfaceTest( "Sawtooth", new SawtoothRandGen( 1000UL ) );

  RNGInterfaceTest( "Square", new SquareRandGen( 10UL, 1010UL, 10UL ) );

  RNGInterfaceTest( "Counter", new CounterRandGen( 57UL, 2UL ) );

  RNGInterfaceTest( "Compiler", new CompilerRandGen( 817UL ) );

  RNGInterfaceTest( "SHA Counter",
                    new SHACtrRandGen( "This is a test of SHA.", 5UL ) );

  RNGInterfaceTest( "Mitchell - Moore",
                    new MitchellMooreRandGen( 0xffffffffUL,
                                              new Knuth28RandGen( 123UL ) ) );

  RNGInterfaceTest( "Linear multiply with carry",
                    new LinMultWCarryRandGen( 0x12345678UL, 123456UL, 923UL ) );

  return 0;
 }



void    RNGInterfaceTest( char *pszLabel, RandomGenerator *pRNG )
 {
  int              i,j;
  unsigned         RNarray[50];
  unsigned         RNarray2[50];
  _STRING          InitString;
  RandGenToken     Token;
  RandomGenerator *pClone;
  strstream        TokenStream;

  cout << pszLabel << ":" << endl; enter();

  if(pRNG->bad())
   {
    cout << "Error: Generator is bad.\n" << endl;
    return;
   }

  for(i = 0;i<50;i++)
   {
    RNarray[i] = (unsigned)pRNG->LimitedNumber( 1000L );
   }
  cout << "Create token" << endl; enter();
  Token = pRNG->CreateToken();

  InitString = Token.CreateInitString();
  cout << "\tInit String = " << InitString << endl;

  cout << "\tToken = " << Token << endl; enter();
  TokenStream << Token;
  TokenStream.seekg( (streampos)0, ios::beg );

  for(i = 0;i<50;i++)
   {
    RNarray2[i] = (unsigned)pRNG->LimitedNumber( 1000L );
   }

  if(!pRNG->SetToken( Token ))
   {
    cout << "SetToken failed" << endl;
    return;
   }
  pClone = pRNG->Clone();

  cout << "\tSet token" << endl; enter();
  for(i = 0;i<50;i++)
   {
    if(RNarray2[i] != (unsigned)pRNG->LimitedNumber( 1000L ))
      cout << "Error in #" << i << endl;
   }

  pRNG->Reset();

  cout << "\tReset" << endl; enter();
  for(i = 0;i<50;i++)
   {
    if(RNarray[i] != (unsigned)pRNG->LimitedNumber( 1000L ))
      cout << "Error in #" << i << endl;
   }

  if(!Token.Initialize( InitString ))
   {
    cout << "Initialize Token failed" << endl;
    return;
   }

  if(!pRNG->SetToken( Token ))
   {
    cout << "SetToken failed" << endl;
    return;
   }

  cout << "\tInitialize token" << endl; enter();
  for(i = 0;i<50;i++)
   {
    if(RNarray[i] != (unsigned)pRNG->LimitedNumber( 1000L ))
      cout << "Error in #" << i << endl;
   }

  cout << "\tToken from stream" << endl; enter();
  TokenStream >> Token;

  if(!pRNG->SetToken( Token ))
   {
    cout << "SetToken failed" << endl;
    return;
   }

  cout << "\tRebuild token" << endl; enter();
  for(i = 0;i<50;i++)
   {
    RNarray2[i] = (unsigned)pRNG->LimitedNumber( 1000L );
   }

  cout << "\tUsing clone" << endl; enter();
  for(i = 0;i<50;i++)
   {
    RNarray2[i] = (unsigned)pClone->LimitedNumber( 1000L );
   }

  cout << "\tRaw numbers" << endl; enter();
  for(i=0;i < 5;++i)
   {
    for(j=0;j < 6;++j)
     {
      cout << setw(12) << pRNG->Number();
     }
    cout << endl;
   }

  cout << "\tFloating output" << endl; enter();
  for(i=0;i < 5;++i)
   {
    for(j=0;j < 6;++j)
     {
      cout << setw(12) << pRNG->FloatNumber();
     }
    cout << endl;
   }

  cout << "\tDeleting..."; enter();
  delete    pClone;
  delete    pRNG;

  cout << "Complete" << endl;
 }
