/* =========================================================================
 *                             RANDADPT.CPP
 * -------------------------------------------------------------------------
 *  Implementation file for a random number generator which changes the
 *  output of another RNG as an object.
 *
 *  Version 0.9
 *
 *  Copyright 1995-1997 by G. Wade Johnson
 *   Use of this code is placed in the public domain as long as the
 *   copyright notice is retained.
 */

#include <string.h>
#include <memory.h>
#include "environ.h"

#include _STRSTREAM_HEADER

#include "randadpt.h"




class  AdaptorRandGenToken : public RandGenTokenImpl
 {
public:
  virtual   ~AdaptorRandGenToken( void )
   { }

  virtual _STRING   CreateInitString( void ) const;
  virtual int   Initialize( istream &InitStream );
  virtual DIFFRET(RandGenTokenImpl,AdaptorRandGenToken) *Clone() const
   {
    return new AdaptorRandGenToken( myToken );
   }

  void       Print( ostream &os ) const
   {
    os << '(' << myToken << ')' << flush;
   }
  void       Read( istream &is )
   {
    char  dummy;
    is >> dummy >> myToken >> dummy;
   } 

private:
  friend class AdaptorRandGen;
  AdaptorRandGenToken( const RandGenToken &Tok ) : myToken(Tok)
   { }

  RandGenToken myToken;
 } ;




_STRING   AdaptorRandGenToken::CreateInitString( void ) const
 {
  ostrstream  String;

  String << '(' << myToken.CreateInitString() << ')' << ends;

  return _STRING( String.str() );
 }






int   AdaptorRandGenToken::Initialize( istream &InitStream )
 {
  char    junk;

  InitStream >> ws >> junk;

  if(!myToken.Initialize( InitStream ))
    return 0;

  InitStream >> ws >> junk;

  return 1;
 }






AdaptorRandGen::AdaptorRandGen( RandomGenerator *pGen ) : mypRandGen(pGen)
 {
  AdaptorRandGen::Reset();
 }




AdaptorRandGen::~AdaptorRandGen()
 { 
  delete mypRandGen;
 }


 
RandGenToken  AdaptorRandGen::CreateToken( void ) const
 {
  return RandGenToken( new AdaptorRandGenToken( mypRandGen->CreateToken() ) );
 }



int     AdaptorRandGen::SetToken( const RandGenToken &Token )
 {
  const AdaptorRandGenToken *pSRGTok =
             DYNACAST( const AdaptorRandGenToken * )(Token.Implementation());

  if(pSRGTok)
    return mypRandGen->SetToken( pSRGTok->myToken );

  return 0;
 }


void        AdaptorRandGen::Reset( void )
 {
  mypRandGen->Reset();
 }




void        AdaptorRandGen::IntSeed( size_rand seed )
 {
  mypRandGen->IntSeed( seed );
  Reset();
 }
