/* =========================================================================
 *                               RANDADPT.H
 * -------------------------------------------------------------------------
 *  Interface file for a random number generator which modifies the
 *  output of another RNG as an object.
 *
 *  Version 0.9
 *
 *  Copyright 1995-1997 by G. Wade Johnson
 *   Use of this code is placed in the public domain as long as the
 *   copyright notice is retained.
 */

#if !defined( RANDADPT_H )
  #define  RANDADPT_H

  #if !defined( RANDGEN_H )
    #include "randgen.h"
  #endif


  class  AdaptorRandGen : public RandomGenerator
   {
    RandomGenerator * const mypRandGen;

  private:
    // protect the = operator and copy constructor so they cannot be used.
    AdaptorRandGen& operator=( AdaptorRandGen const & );
    AdaptorRandGen( AdaptorRandGen const & );

  protected:
    RandomGenerator *pRandGen( void ) const  { return mypRandGen; }

  public:
    EXPLICIT AdaptorRandGen( RandomGenerator *pGen );
    virtual   ~AdaptorRandGen();
 
    virtual RandGenToken  CreateToken( void ) const ;
    virtual int     SetToken( const RandGenToken &Token );

    virtual  void        Reset( void );

    virtual  void        IntSeed( size_rand seed );
    virtual  size_rand   Number( void )=0;
    virtual  size_rand   MaxRandom( void ) const
     {
      return pRandGen()->MaxRandom();
     }
   } ;


  class A15BitRandGen : public AdaptorRandGen
   {
  private:
    // protect the = operator and copy constructor so they cannot be used.
    A15BitRandGen& operator=( const A15BitRandGen& );
    A15BitRandGen( const A15BitRandGen& );

  public:
    A15BitRandGen( RandomGenerator *pGen ) : AdaptorRandGen( pGen )
     { }
    virtual   ~A15BitRandGen() { }
 
    virtual  void       IntSeed( size_rand seed )
     {
      pRandGen()->IntSeed( (seed & 0x7fff) );
     }
    virtual  size_rand   Number( void )
     {
      return ((unsigned)(pRandGen()->Number() >> 16) & 0x7fff);
     }
    virtual  DIFFRET(RandomGenerator,A15BitRandGen)  *ReCreate( void ) const
     {
      return new A15BitRandGen( pRandGen()->ReCreate() );
     }

    virtual  size_rand   MaxRandom( void ) const { return 0x7fff; }
   } ;
   
#endif
