/* =========================================================================
 *                               RANDBTBL.CPP
 * -------------------------------------------------------------------------
 *  Implementation file for a byte-stream table driven pseudo-random number
 *  generator as an object.  (Assumes table on disk.)
 *
 *  Version 0.9
 *
 *  Copyright 1995-1997 by G. Wade Johnson
 *   Use of this code is placed in the public domain as long as the
 *   copyright notice is retained.
 */

#include <ctype.h>
#include <string.h>
#include "environ.h"
#include "randbtbl.h"





ByteTableRandGen::ByteTableRandGen( const char *szFileName, streampos start ) :
  TextTableRandGen( szFileName, start )
 { }




size_rand    ByteTableRandGen::Number( void )
 {
  return LimitedNumber( (size_rand)0 );
 }



size_rand    ByteTableRandGen::LimitedNumber( size_rand limit )
 {
  size_rand  NumAttempt;

  if(limit == 0)
   {
    NumAttempt = getNumber( 10 );
   }
  else
   {
    unsigned   NumDigits = getNumberDigits( limit );

    do
     {
      NumAttempt = getNumber( NumDigits );
     } while(NumAttempt >= limit);
   }

  return NumAttempt;
 }





DIFFRET(RandomGenerator,DecimalTableRandGen)
                                   *DecimalTableRandGen::ReCreate( void ) const
 {
  return new DecimalTableRandGen( FileName(), Start() );
 }




size_rand    DecimalTableRandGen::getNumber( unsigned digits )
 {
  ifstream  &is = Table();
  size_rand  attempt = 0;
  size_rand  newattempt = 0;
  char       aDigit;

  if(!Table())
    return 0;

  for(;digits > 0;digits--)
   {
    newattempt = attempt * 10;
    if(newattempt < attempt)
      return attempt;
    attempt = newattempt;

    do
     {
      is >> aDigit;

      if(is.eof())
       {
        InitTable();
        is >> aDigit;
       }

     } while(!isdigit( aDigit ) );

    newattempt += (aDigit - '0');          // Assumes ASCII character set
    if(newattempt < attempt)
      return attempt;
    attempt = newattempt;
   }

  return attempt;
 }







unsigned   DecimalTableRandGen::getNumberDigits( size_rand limit )
 {
  if(limit <= 10) return 1;
  if(limit <= 100) return 2;
  if(limit <= 1000) return 3;
  if(limit <= 10000) return 4;
  if(limit <= 100000L) return 5;
  if(limit <= 1000000L) return 6;
  if(limit <= 10000000L) return 7;
  if(limit <= 100000000L) return 8;
  if(limit <= 1000000000L) return 9;
  return 10;
 }



DIFFRET(RandomGenerator,HexTableRandGen)
                                       *HexTableRandGen::ReCreate( void ) const
 {
  return new HexTableRandGen( FileName(), Start() );
 }




size_rand    HexTableRandGen::getNumber( unsigned digits )
 {
  ifstream  &is = Table();
  size_rand  attempt = 0;
  size_rand  newattempt = 0;
  char       aDigit;

  if(!Table())
    return 0;

  for(;digits > 0;digits--)
   {
    newattempt = attempt * 16;
    if(newattempt < attempt)
      return attempt;
    attempt = newattempt;

    do
     {
      is >> aDigit;

      if(is.eof())
       {
        InitTable();
        is >> aDigit;
       }

     } while(!isxdigit( aDigit ) );

    if('0' <= aDigit && aDigit <= '9')
      newattempt += (aDigit - '0');          // Assumes ASCII character set
    else if('a' <= aDigit && aDigit <= 'f')
      newattempt += (aDigit - 'a' + 10);     // Assumes ASCII character set
    else if('A' <= aDigit && aDigit <= 'F')
      newattempt += (aDigit - 'A' + 10);     // Assumes ASCII character set

    if(newattempt < attempt)
      return attempt;
    attempt = newattempt;
   }

  return attempt;
 }







unsigned   HexTableRandGen::getNumberDigits( size_rand  limit )
 {
  if(limit <= 0x10) return 1;
  if(limit <= 0x100) return 2;
  if(limit <= 0x1000) return 3;
  if(limit <= 0x10000L) return 4;
  if(limit <= 0x100000L) return 5;
  if(limit <= 0x1000000L) return 6;
  if(limit <= 0x10000000L) return 7;
  return 8;
 }
