/* =========================================================================
 *                                 RANDBTBL.H
 * -------------------------------------------------------------------------
 *  Interface file for a byte-stream table driven random number generator
 *  as an object.
 *  (Assumes that the table resides on disk.)
 *
 *  Version 0.9
 *
 *  Copyright 1995-1997 by G. Wade Johnson
 *   Use of this code is placed in the public domain as long as the
 *   copyright notice is retained.
 */

#if !defined( RANDBTBL_H )
  #define RANDBTBL_H

  #if !defined( ENVIRON_H )
    #include "environ.h"
  #endif
  #include _FSTREAM_HEADER
  #if !defined( RANDGEN_H )
    #include "randgen.h"
  #endif
  #if !defined( RANDTABL_H )
    #include "randtabl.h"
  #endif

  class  ByteTableRandGen : public TextTableRandGen
   {
  private:
    // protect the = operator and copy constructor so they cannot be used.
    ByteTableRandGen& operator=( ByteTableRandGen const & );
    ByteTableRandGen( ByteTableRandGen const & );

  public:
    ByteTableRandGen( const char *szFileName, streampos start );
    virtual   ~ByteTableRandGen() { }
 
    virtual  size_rand   Number( void );
    virtual  size_rand   LimitedNumber( size_rand limit );

  protected:
    virtual  size_rand   getNumber( unsigned digits )=0;
    virtual  unsigned    getNumberDigits( size_rand limit )=0;
   } ;



  class  DecimalTableRandGen : public ByteTableRandGen
   {
  private:
    // protect the = operator and copy constructor so they cannot be used.
    DecimalTableRandGen& operator=( DecimalTableRandGen const & );
    DecimalTableRandGen( DecimalTableRandGen const & );

  public:
    EXPLICIT DecimalTableRandGen( const char *szFileName, streampos start=0 ) :
      ByteTableRandGen( szFileName, start )  { }

    virtual DIFFRET(RandomGenerator,DecimalTableRandGen)
                                                      *ReCreate( void ) const;

  protected:
    virtual  size_rand   getNumber( unsigned digits );
    virtual  unsigned    getNumberDigits( size_rand limit );
   } ;



  class  HexTableRandGen : public ByteTableRandGen
   {
  private:
    // protect the = operator and copy constructor so they cannot be used.
    HexTableRandGen& operator=( HexTableRandGen const & );
    HexTableRandGen( HexTableRandGen const & );

  public:
    EXPLICIT HexTableRandGen( const char *szFileName, streampos start=0 ) :
      ByteTableRandGen( szFileName, start )  { }

    virtual DIFFRET(RandomGenerator,HexTableRandGen)  *ReCreate( void ) const;

  protected:
    virtual  size_rand   getNumber( unsigned digits );
    virtual  unsigned    getNumberDigits( size_rand limit );
   } ;


#endif
