/* =========================================================================
 *                              RANDCOMB.CPP
 * -------------------------------------------------------------------------
 *  Implementation file for a random number generator which combines the
 *  outputs of to other RNGs as an object.
 *
 *  Version 0.9
 *
 *  Copyright 1995-1997 by G. Wade Johnson
 *   Use of this code is placed in the public domain as long as the
 *   copyright notice is retained.
 */

#include <string.h>
#include <memory.h>
#include "environ.h"

#include _STRSTREAM_HEADER

#include "randcomb.h"




class  CombineRandGenToken : public RandGenTokenImpl
 {
public:
  virtual   ~CombineRandGenToken( void )
   { }

  virtual _STRING CreateInitString( void ) const;
  virtual int   Initialize( istream &InitStream );
  virtual DIFFRET(RandGenTokenImpl,CombineRandGenToken)   *Clone() const
   {
    return new CombineRandGenToken( myTokenX, myTokenY );
   }

  void       Print( ostream &os ) const
   {
    os << '(' << myTokenX << ',' << myTokenY << ')' << flush;
   }
  void       Read( istream &is )
   {
    char  dummy;
    is >> dummy >> myTokenX >> dummy >> myTokenY >> dummy;
   } 

private:
  friend class CombineRandGen;
  CombineRandGenToken( const RandGenToken &TokX, const RandGenToken &TokY ) :
    myTokenX(TokX), myTokenY(TokY)
   { }

  RandGenToken  myTokenX;
  RandGenToken  myTokenY;
 } ;



_STRING CombineRandGenToken::CreateInitString( void ) const
 {
  ostrstream  String;

  String << '(' << myTokenX.CreateInitString()
         << ',' << myTokenY.CreateInitString()
         << ')' << ends;

  return _STRING( String.str() );
 }






int   CombineRandGenToken::Initialize( istream &InitStream )
 {
  char    junk;

  InitStream >> ws >> junk;

  if(!myTokenX.Initialize( InitStream ))
    return 0;

  InitStream >> ws >> junk;

  if(!myTokenY.Initialize( InitStream ))
    return 0;

  InitStream >> ws >> junk;

  return 1;
 }






CombineRandGen::CombineRandGen( RandomGenerator *pGenX,
                                                   RandomGenerator *pGenY ) :
   mypRandGenX(pGenX), mypRandGenY(pGenY)
 {
  CombineRandGen::Reset();
 }




CombineRandGen::~CombineRandGen()
 { 
  if(mypRandGenX != mypRandGenY)
    delete mypRandGenY;

  delete mypRandGenX;
  mypRandGenY = mypRandGenX = 0;
 }





RandGenToken   CombineRandGen::CreateToken( void ) const
 {
  return RandGenToken(
                 new CombineRandGenToken( mypRandGenX->CreateToken(),
                                          mypRandGenY->CreateToken()
                                        )
                     );
 }



int     CombineRandGen::SetToken( const RandGenToken &Token )
 {
  const CombineRandGenToken *pSRGTok =
             DYNACAST( const CombineRandGenToken * )(Token.Implementation());

  if(pSRGTok)
   {
    return mypRandGenX->SetToken( pSRGTok->myTokenX ) &&
           mypRandGenY->SetToken( pSRGTok->myTokenY );
   }

  return 0;
 }


void        CombineRandGen::Reset( void )
 {
  mypRandGenX->Reset();
  mypRandGenY->Reset();
 }



void        CombineRandGen::IntSeed( size_rand seed )
 {
  mypRandGenX->IntSeed( seed );
  mypRandGenY->IntSeed( seed );

  mypRandGenX->Reset();
  mypRandGenY->Reset();
 }
