/* =========================================================================
 *                               RANDCOMB.H
 * -------------------------------------------------------------------------
 *  Interface file for a random number generator which combines the
 *  outputs of to other RNGs as an object.
 *
 *  Version 0.9
 *
 *  Copyright 1995-1997 by G. Wade Johnson
 *   Use of this code is placed in the public domain as long as the
 *   copyright notice is retained.
 */

#if !defined( RANDCOMB_H )
  #define  RANDCOMB_H

  #if !defined( RANDGEN_H )
    #include "randgen.h"
  #endif


  class  CombineRandGen : public RandomGenerator
   {
    RandomGenerator *mypRandGenX;
    RandomGenerator *mypRandGenY;

  private:
    // protect the = operator and copy constructor so they cannot be used.
    CombineRandGen& operator=( CombineRandGen const & );
    CombineRandGen( CombineRandGen const & );

  protected:
    RandomGenerator *pRandGenX( void ) const  { return mypRandGenX; }
    RandomGenerator *pRandGenY( void ) const  { return mypRandGenY; }

  public:
    CombineRandGen( RandomGenerator *pGenX, RandomGenerator *pGenY );
    virtual   ~CombineRandGen();

    virtual RandGenToken  CreateToken( void ) const;
    virtual int     SetToken( const RandGenToken &Token );

    virtual  void        Reset( void );

    virtual  void        IntSeed( size_rand );
    virtual  size_rand   Number( void )=0;
    virtual  size_rand   MaxRandom( void ) const
     {
      return pRandGenX()->MaxRandom();
     }
   } ;
   
#endif
