#!perl
# ========================================================================
#                               RANDFILE.PL
# ------------------------------------------------------------------------
#  Create a program to generate the random number files used by the DIEHARD
#   test suite.
#
#  Version 0.2
#
#  Copyright 1996-1997 by G. Wade Johnson
#   Use of this code is placed in the public domain as long as the
#   copyright notice is retained.
#

%Fields = ( 'Generator', 'Choose', 'Parameters', '1' );
@FieldKeys = ( '', 'Generator', 'Parameters' ); # never use the first one.

$OutputFile = "randfile.cpp";

for(;;)
 {
  print "\n"x25;  # attempt to clear screen the hard way.
  print "Build RANDFILE.CPP\n\n";

  for($i=1;$i < scalar(@FieldKeys);$i++)
   {
    print " $i\t$FieldKeys[$i]\t$Fields{$FieldKeys[$i]}\n"
   }
  print "\nSelect a parameter or 0 to create the file: ";

  chop($input = <>);
  exit 0 if("" eq $input);   # quit on empty entry.

  $select = 0 + $input;
  last if(0 == $select);

  next if($select < 0 || $#FieldKeys < $select);

  print "\nEnter new value for \"$FieldKeys[$select]\": ";
  chop($input = <>);
  $Fields{$FieldKeys[$select]} = $input;
 }

open( OUT, "> $OutputFile" ) || die "Unable to open $OutputFile.\n";

while(<DATA>)
 {
  s/\$\$(\S+)\$\$/$Fields{"$1"}/g;
  print OUT;
 }

close( OUT );
print "Output written to $OutputFile.\n";

__END__
/* ========================================================================
 *                               RANDFILE.CPP
 * ------------------------------------------------------------------------
 *  Generate the random number files required by the DIEHARD tests.
 *
 *  Copyright 1995-1996 by G. Wade Johnson
 *   Use of this code is placed in the public domain as long as the
 *   copyright notice is retained.
 */


#include <stdlib.h>
#include "environ.h"

#include _IOSTREAM_HEADER
#include _FSTREAM_HEADER
#include _STRSTREAM_HEADER

#include "rand_all.h"
#include "erndmcnm.h"
#include "erndsecm.h"
#include "erndlcon.h"
#include "erndmcon.h"

#define  FILE_SIZE   11468800L
#define  NUM_NUMBERS (FILE_SIZE/sizeof(size_rand))


int    main( void )
 {
  char    szFilename[80];

  RandomGenerator  *pRNG = new $$Generator$$( $$Parameters$$ );

  cout << "Output filename: " << flush;
  cin >> szFilename;

#if defined( _HAS_BINARY )
  ofstream fos( szFilename, ios::out|ios::binary );
#else
  ofstream fos( szFilename, ios::out );
#endif
  if(fos)
   {
    cout << "Build output file for the generator\n"
         << "$$Generator$$( $$Parameters$$ )\n"
         << "Working........" << endl;

    long        l = 0L;
    size_rand   rnumber;
    for(l=0L;l < NUM_NUMBERS;l++)
     {
      rnumber = pRNG->Number();

      fos.write( (char *)(&rnumber), sizeof( size_rand ) );
     }
   }
  else
   {
    cout << "Cannot open \"" << szFilename << "\" for output.\n";
   }

  delete pRNG;

  return 0;
 }
