/* ========================================================================
 *                                RANDFUNC.H
 * ------------------------------------------------------------------------
 *  Function Object interface for the PRNG class library.
 *
 *  Version 0.9
 *
 *  Copyright 1995-1997 by G. Wade Johnson
 *   Use of this code is placed in the public domain as long as the
 *   copyright notice is retained.
 */

#if !defined( RANDFUNC_H )
  #define RANDFUNC_H

  #if !defined( RANDGEN_H )
    #include "randgen.h"
  #endif

  class RNGFuncAdaptor
   {
  private:
    RandomGenerator  *myRNG;

  private:
    // protect the = operator and copy constructor so they cannot be used.
    RNGFuncAdaptor( RNGFuncAdaptor const &rhs );
    RNGFuncAdaptor& operator=( RNGFuncAdaptor const &rhs );

  public:
    EXPLICIT RNGFuncAdaptor( RandomGenerator *pRNG ) : myRNG(pRNG)
     { }
    ~RNGFuncAdaptor( void ) { delete myRNG; }

    size_rand   operator()( void )
     {
      return myRNG->Number();
     }
   } ;


  class RNGFuncAdaptorFloat
   {
  private:
    RandomGenerator  *myRNG;

  private:
    // protect the = operator and copy constructor so they cannot be used.
    RNGFuncAdaptorFloat( RNGFuncAdaptorFloat const &rhs );
    RNGFuncAdaptorFloat& operator=( RNGFuncAdaptorFloat const &rhs );

  public:
    EXPLICIT RNGFuncAdaptorFloat( RandomGenerator *pRNG ) : myRNG(pRNG)
     { }
    ~RNGFuncAdaptorFloat( void ) { delete myRNG; }

    double      operator()( void )
     {
      return myRNG->FloatNumber();
     }
   } ;


  class RNGFuncAdaptorLimit
   {
  private:
    RandomGenerator  *myRNG;
    const size_rand    myLimit;

  private:
    // protect the = operator and copy constructor so they cannot be used.
    RNGFuncAdaptorLimit( RNGFuncAdaptorLimit const &rhs );
    RNGFuncAdaptorLimit& operator=( RNGFuncAdaptorLimit const &rhs );

  public:
    RNGFuncAdaptorLimit( RandomGenerator *pRNG, size_rand limit ) :
     myRNG(pRNG), myLimit(limit)
     { }
    ~RNGFuncAdaptorLimit( void ) { delete myRNG; }

    size_rand   operator()( void )
     {
      return myRNG->LimitedNumber( myLimit );
     }

    size_rand   operator()( size_rand limit )
     {
      return myRNG->LimitedNumber( limit );
     }
   } ;


  class RNGFuncAdaptorPercentLikely
   {
  private:
    RandomGenerator  *myRNG;
    const unsigned     myPercent;

  private:
    // protect the = operator and copy constructor so they cannot be used.
    RNGFuncAdaptorPercentLikely( RNGFuncAdaptorPercentLikely const &rhs );
    RNGFuncAdaptorPercentLikely& operator=( RNGFuncAdaptorPercentLikely const &rhs );

  public:
    RNGFuncAdaptorPercentLikely( RandomGenerator *pRNG, unsigned percent )
     : myRNG(pRNG), myPercent(percent)
     { }
    ~RNGFuncAdaptorPercentLikely( void ) { delete myRNG; }

    BOOL   operator()( void )
     {
      return (myPercent > (unsigned)myRNG->LimitedNumber( 100UL ));
     }

    BOOL   operator()( unsigned percent )
     {
      return (percent > (unsigned)myRNG->LimitedNumber( 100UL ));
     }
   } ;



  class RNGFuncAdaptorRange
   {
  private:
    RandomGenerator  *myRNG;
    const size_rand    myLower;
    const size_rand    myRange;

  private:
    // protect the = operator and copy constructor so they cannot be used.
    RNGFuncAdaptorRange( RNGFuncAdaptorRange const &rhs );
    RNGFuncAdaptorRange& operator=( RNGFuncAdaptorRange const &rhs );

  public:
    RNGFuncAdaptorRange( RandomGenerator *pRNG, size_rand lower,
                          size_rand upper ) :
     myRNG(pRNG), myLower(lower), myRange(upper-lower+1)
     { }
    ~RNGFuncAdaptorRange( void ) { delete myRNG; }

    size_rand operator()( void )
     {
      return myLower + myRNG->LimitedNumber( myRange );
     }
   } ;

#endif
