/* =========================================================================
 *                                 RANDICON.H
 * -------------------------------------------------------------------------
 *  Interface file for a inverse congruential pseudo-random number generator
 *  with a modulus of 2^32-1 as an object. As described in Andreas
 *  Weingartner's Master's thesis.
 *
 *  Version 0.9
 *
 *  Copyright 1995-1997 by G. Wade Johnson
 *   Use of this code is placed in the public domain as long as the
 *   copyright notice is retained.
 */

#if !defined( RANDICON_H )
  #define RANDICON_H

  #if !defined( RANDGEN_H )
    #include "randgen.h"
  #endif
  #if !defined( RANDCONG_H )
    #include "randcong.h"
  #endif


  class  InvConModRandGen : public CongruentialRandGen
   {
  private:
    const unsigned long   myMultiplier;
    const unsigned long   myIncrement;
    const unsigned long   myModulus;

  private:
    // protect the = operator and copy constructor so they cannot be used.
    InvConModRandGen& operator=( InvConModRandGen const & );
    InvConModRandGen( InvConModRandGen const & );

  protected:
    unsigned long       Multiplier( void ) const  { return myMultiplier; }
    unsigned long       Increment( void )  const  { return myIncrement; }
    unsigned long       Modulus( void )    const  { return myModulus; }
    virtual size_rand   InvertSeed( void ) const;

  public:
    InvConModRandGen( unsigned long mult, unsigned long incr,
                      unsigned long mod, size_rand seed=1 )
     : CongruentialRandGen(seed), myMultiplier(mult), myIncrement(incr),
      myModulus(mod)
     {
      SetSeed( seed % Modulus() );
      CongruentialRandGen::Reset();
     }
    virtual   ~InvConModRandGen() { }
 
    virtual DIFFRET(RandomGenerator,InvConModRandGen)  *ReCreate( void ) const;

    virtual  size_rand   Number( void );
    virtual  size_rand   MaxRandom( void ) const { return Modulus()-1; }
   } ;


  class  InvConRandGen : public InvConModRandGen
   {
  public:
    InvConRandGen( unsigned long mult, unsigned long incr, size_rand seed=1 )
     : InvConModRandGen( mult, incr, 0xFFFFFFFBUL, seed )
     { }
   } ;


#endif
