/* =========================================================================
 *                                 RANDLCON.H
 * -------------------------------------------------------------------------
 *  Interface file for a linear congruential pseudo-random number generator
 *  with a modulus of 2^32 as an object.
 *
 *  Version 0.9
 *
 *  Copyright 1995-1997 by G. Wade Johnson
 *   Use of this code is placed in the public domain as long as the
 *   copyright notice is retained.
 */

#if !defined( RANDLCON_H )
  #define RANDLCON_H

  #if !defined( RANDGEN_H )
    #include "randgen.h"
  #endif
  #if !defined( RANDCONG_H )
    #include "randcong.h"
  #endif


  class  LinConRandGen : public CongruentialRandGen
   {
  private:
    const unsigned long   myMultiplier;
    const unsigned long   myIncrement;

  private:
    // protect the = operator and copy constructor so they cannot be used.
    LinConRandGen& operator=( LinConRandGen const & );
    LinConRandGen( LinConRandGen const & );

  protected:
    unsigned long   Multiplier( void ) const  { return myMultiplier; }
    unsigned long   Increment( void )  const  { return myIncrement; }

  public:
    LinConRandGen( unsigned long mult, unsigned long incr, size_rand seed=1 ) :
      CongruentialRandGen(seed), myMultiplier(mult), myIncrement(incr)
     { }
    virtual   ~LinConRandGen() { }
 
    virtual DIFFRET(RandomGenerator,LinConRandGen)  *ReCreate( void ) const;

    virtual  size_rand   Number( void );
   } ;

#endif
