/* =========================================================================
 *                                 randlfib.H
 * -------------------------------------------------------------------------
 *  Interface file for Lagged Fibonacci random number generator as an object.
 *
 *  Version 0.9
 *
 *  Copyright 1996-1997 by G. Wade Johnson
 *   Use of this code is placed in the public domain as long as the
 *   copyright notice is retained.
 */

#if !defined( RANDLFIB_H )
  #define RANDLFIB_H

  #if !defined( RANDGEN_H )
    #include "randgen.h"
  #endif
  #if !defined( RINGBUFF_H )
    #include "ringbuff.h"
  #endif


  class  LaggedFibonacciRandGen : public RandomGenerator
   {
  private:
    const unsigned          myUpperTap;
    const unsigned          myLowerTap;
    RingBuffer              myBuffer;
    RingBuffer::Iterator    myHigh;
    RingBuffer::Iterator    myLow;
    RandomGenerator        *myInitializer;

  private:
    // protect the = operator and copy constructor so they cannot be used.
    LaggedFibonacciRandGen& operator=( LaggedFibonacciRandGen const & );
    LaggedFibonacciRandGen( LaggedFibonacciRandGen const & );

  protected:
    // access methods for private data.
    unsigned    UpperTap()  const { return myUpperTap; }
    unsigned    LowerTap()  const { return myLowerTap; }
    size_rand   LowValue()  const { return *myLow; }
    size_rand   HighValue() const { return *myHigh; }
    RandomGenerator  *Initializer() const { return myInitializer; }

    void        IncrTaps( void );
    void        SetNewElement( size_rand val )
     {
      myBuffer.add( val );
     }


    // initializer methods for private data.
    void     InitArray( void );
    void     InitTaps( void )
     {
      myHigh = myBuffer.MakeIterator( 0 );
      myLow  = myBuffer.MakeIterator( myUpperTap - myLowerTap );
     }

  public:
    LaggedFibonacciRandGen( unsigned lower, unsigned upper,
                                                       RandomGenerator *pRng );
    virtual   ~LaggedFibonacciRandGen();

    virtual DIFFRET(RandomGenerator,LaggedFibonacciRandGen)
                                                        *ReCreate( void ) const
     {
      return new LaggedFibonacciRandGen( myLowerTap, myUpperTap,
                                         myInitializer->ReCreate() );
     }

    virtual RandGenToken  CreateToken( void ) const;
    virtual int     SetToken( const RandGenToken &Token );

    virtual void    Reset( void );

    virtual  void        IntSeed( size_rand seed );

    virtual  size_rand   Number( void );
   } ;

#endif
