/* =========================================================================
 *                                RandLMWC.CPP
 * -------------------------------------------------------------------------
 *  Implementation file for simple version of Marsaglia's multiply with carry
 *  generator as an object.
 *
 *  Version 0.9
 *
 *  Copyright 1996-1997 by G. Wade Johnson
 *   Use of this code is placed in the public domain as long as the
 *   copyright notice is retained.
 */

#include "environ.h"

#include _STRSTREAM_HEADER

#include <string.h>

#include "modarith.h"
#include "randlmwc.h"



class  LinMultWCarryRandGenToken : public RandGenTokenImpl
 {
  private:
    // protect the = operator and copy constructor so they cannot be used.
    LinMultWCarryRandGenToken &operator=( const LinMultWCarryRandGenToken & /* rhs */ );
    LinMultWCarryRandGenToken( const LinMultWCarryRandGenToken & );

public:
  virtual   ~LinMultWCarryRandGenToken( void ) { }

  virtual _STRING   CreateInitString( void ) const;
  virtual int   Initialize( istream &InitStream );
  virtual DIFFRET(RandGenTokenImpl,LinMultWCarryRandGenToken)   *Clone() const
   {
    return new LinMultWCarryRandGenToken( myCarry,    myCurrCarry,
                                          myBaseSeed, myCurrSeed );
   }

  void       Print( ostream &os ) const
   {
    // write the token to os

    os << '(' << myCarry << ',' << myCurrCarry << ','
       << myBaseSeed << ',' << myCurrSeed << ')';
   }
  void       Read( istream &is )
   {
    char   dummy;
    // read the token from is

    is >> dummy >> myCarry >> dummy >> myCurrCarry >> dummy
       >> myBaseSeed >> dummy >> myCurrSeed >> dummy;
   }

private:
  friend class LinMultWCarryRandGen;
  LinMultWCarryRandGenToken( size_rand carry, size_rand currCarry,
                             size_rand seed,  size_rand currSeed )
   : myCarry(carry), myCurrCarry(currCarry), myBaseSeed(seed),
     myCurrSeed(currSeed)
   { }

  size_rand         myCarry;
  size_rand         myCurrCarry;
  size_rand         myBaseSeed;
  size_rand         myCurrSeed;
 } ;





_STRING   LinMultWCarryRandGenToken::CreateInitString( void ) const
 {
  ostrstream  aString;

  // write your init string to String.

  aString << '(' << myCarry << ',' << myBaseSeed << ')' << ends;

  return _STRING( aString.str() );
 }




int   LinMultWCarryRandGenToken::Initialize( istream &InitStream )
 {
  // Initialize the token from InitStream
  char   dummy;

  InitStream >> dummy >> myCarry >> dummy >> myBaseSeed >> dummy;

  myCurrCarry = myCarry;
  myCurrSeed  = myBaseSeed;

  return 1;
 }



RandGenToken    LinMultWCarryRandGen::CreateToken( void ) const
 {
  return  RandGenToken(
          new LinMultWCarryRandGenToken( Carry(), CurrCarry(),
                                         Seed(),  CurrSeed() )
                      );
 }



int   LinMultWCarryRandGen::SetToken( const RandGenToken &tok )
 {
  const LinMultWCarryRandGenToken  *pToken =
         DYNACAST( const LinMultWCarryRandGenToken * )(tok.Implementation());

  if(pToken)
   {
    // Reset my data from the token
    SetCarry( pToken->myCarry );
    SetCurrCarry( pToken->myCurrCarry );
    SetSeed( pToken->myBaseSeed );
    SetCurrSeed( pToken->myCurrSeed );

    return 1;
   }

  return 0;
 }





LinMultWCarryRandGen::LinMultWCarryRandGen( size_rand mult, size_rand carry,
                                                    size_rand seed )
 : CongruentialRandGen(seed), myMultiplier(mult),
   myCarry(carry), myCurrCarry(carry)
 { }




//  This is where the rest of the functionality is defined.


size_rand   LinMultWCarryRandGen::Number( void )
 {
  //  generate your next number here
  mod_result     result;
  unsigned long  c;

  result = mod_mult_c( Multiplier(), CurrSeed() );
  c = result.carry;
  result = mod_add_c( result.result, CurrCarry() );

  SetCurrCarry( result.carry+c );
  SetCurrSeed( result.result );

  return CurrSeed();
 }
