/* =========================================================================
 *                                 RandLMWC.H
 * -------------------------------------------------------------------------
 *  Interface file for simple version of Marsaglia's multiply with carry
 *  generator as an object.
 *
 *  Version 0.9
 *
 *  Copyright 1996-1997 by G. Wade Johnson
 *   Use of this code is placed in the public domain as long as the
 *   copyright notice is retained.
 */

#if !defined( RandLMWC_H )
  #define RandLMWC_H

  #if !defined( RANDGEN_H )
    #include "randgen.h"
  #endif
  #if !defined( RANDCONG_H )
    #include "randcong.h"
  #endif


  class  LinMultWCarryRandGen : public CongruentialRandGen
   {
  private:
    // add private data
    const size_rand   myMultiplier;
    size_rand         myCarry;
    size_rand         myCurrCarry;

  private:
    // protect the = operator and copy constructor so they cannot be used.
    LinMultWCarryRandGen& operator=( LinMultWCarryRandGen const & );
    LinMultWCarryRandGen( LinMultWCarryRandGen const & );

  protected:
    // access methods for private data.
    size_rand   Multiplier()  const { return myMultiplier; }
    size_rand   Carry()       const { return myCarry; }
    size_rand   CurrCarry()   const { return myCurrCarry; }
    void        SetCarry( size_rand carry )     { myCarry = carry; }
    void        SetCurrCarry( size_rand carry ) { myCurrCarry = carry; }

  public:
    EXPLICIT  LinMultWCarryRandGen( size_rand mult, size_rand carry=0,
                                                    size_rand seed=1 );
    virtual   ~LinMultWCarryRandGen() { }

    virtual DIFFRET(RandomGenerator,LinMultWCarryRandGen)
                                                        *ReCreate( void ) const
     {
      return new LinMultWCarryRandGen( Multiplier(), Carry(), Seed() );
     }

    virtual RandGenToken  CreateToken( void ) const;
    virtual int     SetToken( const RandGenToken &Token );

    virtual void    Reset( void )
     {
      CongruentialRandGen::Reset();
      SetCurrCarry( Carry() );
     }

    virtual  void   IntSeed( size_rand seed )
     {
      SetSeed( seed );
      Reset();
     }

    virtual  size_rand   Number( void );
   } ;

#endif
