/* =========================================================================
 *                             RANDNOT.CPP
 * -------------------------------------------------------------------------
 *  Interface files for some not-so-random number generators which may be 
 *  used for various forms of testing.
 *
 *  Version 0.9
 *
 *  Copyright 1995-1997 by G. Wade Johnson
 *   Use of this code is placed in the public domain as long as the
 *   copyright notice is retained.
 */

#include <string.h>
#include "environ.h"

#include _STRSTREAM_HEADER

#include "randnot.h"


class   ConstantRandGenToken : public RandGenTokenImpl
 {
public:
  virtual   ~ConstantRandGenToken( void ) { }

  virtual _STRING  CreateInitString( void ) const;
  virtual int   Initialize( istream &InitStream );
  virtual DIFFRET(RandGenTokenImpl,ConstantRandGenToken)   *Clone() const
   {
    return new ConstantRandGenToken();
   }

  void       Print( ostream &os ) const
   {
    os << ' ' << flush;
   }
  void       Read( istream &is )
   {
    is >> ws ;
   } 

private:
  friend class ConstantRandGen;
  ConstantRandGenToken( void )
   { }
 } ;



 
_STRING  ConstantRandGenToken::CreateInitString( void ) const
 {
  ostrstream  String;

  String << ' ' << ends;

  return _STRING( String.str() );
 }



int   ConstantRandGenToken::Initialize( istream &InitStream )
 {
  InitStream >> ws;

  return 1;
 }


RandGenToken  ConstantRandGen::CreateToken( void ) const
 {
  return RandGenToken( new ConstantRandGenToken );
 }




class   SquareRandGenToken : public RandGenTokenImpl
 {
public:
  virtual   ~SquareRandGenToken( void ) { }

  virtual _STRING   CreateInitString( void ) const;
  virtual int   Initialize( istream &InitStream );
  virtual DIFFRET(RandGenTokenImpl,SquareRandGenToken)   *Clone() const
   {
    return new SquareRandGenToken( myCurrent, myCurrentTime );
   }


  void       Print( ostream &os ) const
   {
    os << '(' << myCurrent << ',' << myCurrentTime << ')' << flush;
   }
  void       Read( istream &is )
   {
    char  dummy;
    is >> dummy >> myCurrent >> dummy >> myCurrentTime >> dummy;
   } 

private:
  friend class SquareRandGen;
  SquareRandGenToken( size_rand curr, unsigned Time ) :
    myCurrent(curr), myCurrentTime(Time)
   { }
  size_rand       myCurrent;
  unsigned        myCurrentTime;
 } ;



_STRING  SquareRandGenToken::CreateInitString( void ) const
 {
  ostrstream  String;

  String << '(' << myCurrent << ',' << myCurrentTime << ')' << ends;

  return _STRING( String.str() );
 }



int   SquareRandGenToken::Initialize( istream &InitStream )
 {
  char   dummy;

  InitStream >> ws >> dummy >> myCurrent >> dummy >> ws >> myCurrentTime;
  InitStream >> dummy;

  myCurrentTime = 0;

  return 1;
 }



DIFFRET(RandomGenerator,SquareRandGen)  *SquareRandGen::ReCreate( void ) const
 {
  return new SquareRandGen( myFirst, mySecond, myDuration );
 }



RandGenToken  SquareRandGen::CreateToken( void ) const
 {
  return RandGenToken( new SquareRandGenToken( myCurrent, myCurrentTime ) );
 }



int     SquareRandGen::SetToken( const RandGenToken &tok )
 {
  const SquareRandGenToken  *pToken =
                 DYNACAST( const SquareRandGenToken * )(tok.Implementation());

  if(pToken)
   {
    myCurrent     = pToken->myCurrent;
    myCurrentTime = pToken->myCurrentTime;

    if(!myCurrentTime)
      myCurrent = myFirst;

    return 1;
   }

  return 0;
 }



size_rand   SquareRandGen::Number( void )
 {
  if(myDuration == myCurrentTime++)
   {
    if(myCurrent == myFirst)
      myCurrent = mySecond;
    else
      myCurrent = myFirst;

    myCurrentTime = 0;
   }

  return myCurrent;
 }
