/* =========================================================================
 *                               RANDNOT.H
 * -------------------------------------------------------------------------
 *  Interface file for some not-so-random number generators which may be 
 *  used for various forms of testing.
 *
 *  Version 0.9
 *
 *  Copyright 1995-1997 by G. Wade Johnson
 *   Use of this code is placed in the public domain as long as the
 *   copyright notice is retained.
 */

#if !defined( RANDNOT_H )
  #define RANDNOT_H

  #if !defined( RANDGEN_H )
    #include "randgen.h"
  #endif
  #if !defined( RNDLCONM_H )
    #include "rndlconm.h"
  #endif
  #if !defined( RANDSECM_H )
    #include "randsecm.h"
  #endif

  class  ConstantRandGen : public RandomGenerator
   {
    size_rand   mySeed;

  private:
    // protect the = operator and copy constructor so they cannot be used.
    ConstantRandGen& operator=( ConstantRandGen const & );
    ConstantRandGen( ConstantRandGen const & );

  public:
    EXPLICIT ConstantRandGen( size_rand seed ) : mySeed(seed) { }
    virtual   ~ConstantRandGen() { }
 
    virtual DIFFRET(RandomGenerator,ConstantRandGen)  *ReCreate( void ) const
     {
      return new ConstantRandGen( mySeed );
     }

    virtual RandGenToken  CreateToken( void ) const;
    virtual int     SetToken( const RandGenToken & /* Token */ )
     {
      return 1;
     }

    virtual  void        IntSeed( size_rand seed ) { mySeed = seed; }
    virtual  size_rand   Number( void ) { return mySeed; }
    virtual  size_rand   LimitedNumber( size_rand limit )
     {
      return (size_rand)(Number()%limit);
     }
   } ;




  class  CounterRandGen : public LinConRandGen
   {
  public:
    EXPLICIT CounterRandGen( size_rand seed=0UL, size_rand incr=1UL ) :
      LinConRandGen(1L,incr,seed)
      { }
   } ;


  class  SawtoothRandGen : public LinConModRandGen
   {
  public:
    EXPLICIT SawtoothRandGen( size_rand limit ) :
      LinConModRandGen(1L,1L,limit,0L)
      { }
   } ;


  class  SquareRandGen : public RandomGenerator
   {
    const size_rand   myFirst;
    const size_rand   mySecond;
    const size_rand   myDuration;
    size_rand         myCurrent;
    unsigned          myCurrentTime;

  private:
    // protect the = operator and copy constructor so they cannot be used.
    SquareRandGen& operator=( SquareRandGen const & );
    SquareRandGen( SquareRandGen const & );

  public:
    SquareRandGen( size_rand first, size_rand second, size_rand dura ) :
      myFirst(first), mySecond(second), myDuration(dura),
      myCurrent(first), myCurrentTime(0)
      { }
    virtual   ~SquareRandGen() { }
 
    virtual DIFFRET(RandomGenerator,SquareRandGen)  *ReCreate( void ) const;

    virtual RandGenToken  CreateToken( void ) const;
    virtual int     SetToken( const RandGenToken &Token );

    virtual void    Reset( void )
     {
      myCurrent = myFirst; myCurrentTime = 0;
     }

    virtual  void        IntSeed( size_rand seed )
     {
      myCurrentTime = (unsigned)seed;
      Reset();
     }
    virtual  size_rand   Number( void );
    virtual  size_rand   LimitedNumber( size_rand limit )
     {
      return (size_rand)(Number()%limit);
     }
   } ;

  class  FibonacciRandGen : public SecConModRandGen
   {
  public:
    EXPLICIT FibonacciRandGen( unsigned long mod, size_rand seed=1 )
      : SecConModRandGen(1L,1L,0L,mod,seed)
      { }
   } ;


#endif
