/* =========================================================================
 *                                 RandQuad.H
 * -------------------------------------------------------------------------
 *  Interface file for Quadratic Congruential Random Number Generator as an
 *  object.
 *
 *  Version 0.9
 *
 *  Copyright 1996-1997 by G. Wade Johnson
 *   Use of this code is placed in the public domain as long as the
 *   copyright notice is retained.
 */

#if !defined( RandQuad_H )
  #define RandQuad_H

  #if !defined( RANDGEN_H )
    #include "randgen.h"
  #endif
  #if !defined( RANDCONG_H )
    #include "randcong.h"
  #endif

  class  QuadConRandGen : public CongruentialRandGen
   {
  private:
    const unsigned long   myQuadMultiplier;
    const unsigned long   myLinMultiplier;
    const unsigned long   myIncrement;

  private:
    // protect the = operator and copy constructor so they cannot be used.
    QuadConRandGen& operator=( QuadConRandGen const & );
    QuadConRandGen( QuadConRandGen const & );

  protected:
    // access methods for private data.
    unsigned long   QuadMultiplier() const { return myQuadMultiplier; }
    unsigned long   LinMultiplier()  const { return myLinMultiplier; }
    unsigned long   Increment()      const { return myIncrement; }

  public:
    QuadConRandGen( unsigned long quad, unsigned long lin, unsigned long incr,
                     size_rand seed=1 )
     : CongruentialRandGen(seed), myQuadMultiplier(quad), myLinMultiplier(lin),
       myIncrement(incr)
     { }

    virtual   ~QuadConRandGen() { }

    virtual DIFFRET(RandomGenerator,QuadConRandGen)  *ReCreate( void ) const
     {
      return new QuadConRandGen( QuadMultiplier(), LinMultiplier(),
                                 Increment(), Seed() );
     }

    virtual  size_rand   Number( void );
   } ;

#endif
