/* =========================================================================
 *                                 RANDSEC.CPP
 * -------------------------------------------------------------------------
 *  Implementation file for any second-order congruential pseudo-random number
 *  generator with a modulus of 2^32 as an object.
 *
 *  Version 0.9
 *
 *  Copyright 1995-1997 by G. Wade Johnson
 *   Use of this code is placed in the public domain as long as the
 *   copyright notice is retained.
 */

#include <string.h>
#include "environ.h"

#include _STRSTREAM_HEADER

#include "randsec.h"


class  SecConRandGenToken : public RandGenTokenImpl
 {
public:
  virtual   ~SecConRandGenToken( void ) { }

  virtual _STRING   CreateInitString( void ) const;
  virtual int   Initialize( istream &InitStream );
  virtual DIFFRET(RandGenTokenImpl,SecConRandGenToken)   *Clone() const
   {
    return new SecConRandGenToken( myBaseSeed, myLastSeed, myCurrSeed );
   }

  void       Print( ostream &os ) const
   {
    os << '(' << myBaseSeed << ',' << myLastSeed << ',' << myCurrSeed << ')'
       << flush;
   }
  void       Read( istream &is )
   {
    char  dummy;
    is >> dummy >> myBaseSeed >> dummy >> myLastSeed >> dummy >> myCurrSeed
       >> dummy;
   } 

private:
  friend class SecConRandGen;
  SecConRandGenToken( size_rand baseSeed, unsigned long lastSeed,
                                                      unsigned long currSeed ) :
    myBaseSeed( baseSeed ), myLastSeed( lastSeed ), myCurrSeed( currSeed )
   { }
  size_rand       myBaseSeed;
  unsigned long   myLastSeed;
  unsigned long   myCurrSeed;
 } ;





_STRING  SecConRandGenToken::CreateInitString( void ) const
 {
  ostrstream  String;

  String << myBaseSeed << ends;

  return _STRING( String.str() );
 }




int   SecConRandGenToken::Initialize( istream &InitStream )
 {
  InitStream >> ws >> myBaseSeed;
  myCurrSeed = myBaseSeed;
  myLastSeed = 0;

  return 1;
 }





DIFFRET(RandomGenerator,SecConRandGen)  *SecConRandGen::ReCreate( void ) const
 {
  return new SecConRandGen( MultCurr(), MultLast(), Increment(), Seed() );
 }



RandGenToken    SecConRandGen::CreateToken( void ) const
 {
  return  RandGenToken(
                new SecConRandGenToken( Seed(), myLastSeed, CurrSeed() )
                      );
 }



int   SecConRandGen::SetToken( const RandGenToken &tok )
 {
  const SecConRandGenToken  *pToken =
                 DYNACAST( const SecConRandGenToken * )(tok.Implementation());

  if(pToken)
   {
    SetSeed( pToken->myBaseSeed );
    SetCurrSeed( pToken->myCurrSeed );
    myLastSeed = pToken->myLastSeed;
    return 1;
   }

  return 0;
 }




size_rand    SecConRandGen::Number( void )
 {
  unsigned long  newSeed;

  newSeed = MultLast() * LastSeed() + MultCurr() * CurrSeed() + Increment();
  SetLastSeed( CurrSeed() );
  SetCurrSeed( newSeed );

  return CurrSeed();
 }
