/* =========================================================================
 *                                 RANDSEC.H
 * -------------------------------------------------------------------------
 *  Interface file for any second-order congruential pseudo-random number
 *  generator with a modulus of 2^32 as an object.
 *
 *  Version 0.9
 *
 *  Copyright 1995-1997 by G. Wade Johnson
 *   Use of this code is placed in the public domain as long as the
 *   copyright notice is retained.
 */

#if !defined( RANDSEC_H )
  #define RANDSEC_H

  #if !defined( RANDGEN_H )
    #include "randgen.h"
  #endif
  #if !defined( RANDCONG_H )
    #include "randcong.h"
  #endif

  class  SecConRandGen : public CongruentialRandGen
   {
  private:
    size_rand             myLastSeed;
    const unsigned long   myMultLast;
    const unsigned long   myMultCurr;
    const unsigned long   myIncrement;

  private:
    // protect the = operator and copy constructor so they cannot be used.
    SecConRandGen& operator=( SecConRandGen const & );
    SecConRandGen( SecConRandGen const & );

  protected:
    size_rand             LastSeed( void ) const  { return myLastSeed; }
    const unsigned long   MultLast( void ) const  { return myMultLast; }
    const unsigned long   MultCurr( void ) const  { return myMultCurr; }
    const unsigned long   Increment( void ) const { return myIncrement; }

    void      SetLastSeed( size_rand seed ) { myLastSeed = seed; }

  public:
    SecConRandGen( unsigned long multLast, unsigned long multCurr,
                                 unsigned long incr, size_rand seed=1 ) :
      CongruentialRandGen(seed), myMultLast(multLast), myMultCurr(multCurr),
      myIncrement(incr)
     {
      SecConRandGen::Reset();
     }
    virtual   ~SecConRandGen() { }
 
    virtual DIFFRET(RandomGenerator,SecConRandGen)  *ReCreate( void ) const;

    virtual RandGenToken  CreateToken( void ) const;
    virtual int     SetToken( const RandGenToken &Token );

    virtual  void        Reset( void )
     {
      SetCurrSeed(Seed());
      SetLastSeed(0UL);
     }

    virtual  size_rand   Number( void );
   } ;


#endif
