/* =========================================================================
 *                                 RANDSECM.H
 * -------------------------------------------------------------------------
 *  Interface file for any second-order congruential pseudo-random number
 *  as an object.
 *
 *  Version 0.9
 *
 *  Copyright 1995-1997 by G. Wade Johnson
 *   Use of this code is placed in the public domain as long as the
 *   copyright notice is retained.
 */

#if !defined( RANDSECM_H )
  #define RANDSECM_H

  #if !defined( RANDGEN_H )
    #include "randgen.h"
  #endif
  #if !defined( RANDSEC_H )
    #include "randsec.h"
  #endif


  class  SecConModRandGen : public SecConRandGen
   {
  private:
    const unsigned long   myModulus;

  private:
    // protect the = operator and copy constructor so they cannot be used.
    SecConModRandGen& operator=( SecConModRandGen const & );
    SecConModRandGen( SecConModRandGen const & );

  protected:
    unsigned long   Modulus( void ) const { return myModulus; }

  public:
    SecConModRandGen( unsigned long multLast, unsigned long multCurr,
                      unsigned long incr, unsigned long mod, size_rand seed=1 )
     : SecConRandGen( multLast, multCurr, incr, seed ), myModulus(mod)
     {
      SetSeed( seed % Modulus() );
      SecConModRandGen::Reset();
     }
    virtual   ~SecConModRandGen() { }
 
    virtual DIFFRET(RandomGenerator,SecConModRandGen)  *ReCreate( void ) const;

    virtual  void        Reset( void )
     {
      SetCurrSeed(Seed()); SetLastSeed(0UL);
     }

    virtual  size_rand   Number( void );
    virtual  size_rand   MaxRandom( void ) const { return Modulus()-1; }
   } ;

#endif
