/* =========================================================================
 *                               RANDSHUF.H
 * -------------------------------------------------------------------------
 *  Implementation file for a shuffling pseudo-random number generator
 *  object. (Algorithm M from Knuth, VOl 2)
 *
 *  Version 0.9
 *
 *  Copyright 1995-1997 by G. Wade Johnson
 *   Use of this code is placed in the public domain as long as the
 *   copyright notice is retained.
 */

#if !defined( RANDSHUF_H )
  #define  RANDSHUF_H

  #if !defined( RANDGEN_H )
    #include "randgen.h"
  #endif
  #if !defined( RANDCOMB_H )
    #include "randcomb.h"
  #endif


  class  ShuffledMRandGen : public CombineRandGen
   {
    const unsigned           mySize;
    size_rand               *pRandArray;
    BOOL                     myBadFlag;

  protected:
    void          MakeBad( void ) { myBadFlag = true; }
    void          ClearBad( void ) { myBadFlag = false; }

  private:
    // protect the = operator and copy constructor so they cannot be used.
    ShuffledMRandGen& operator=( ShuffledMRandGen const & );
    ShuffledMRandGen( ShuffledMRandGen const & );

  public:
    ShuffledMRandGen( unsigned size, RandomGenerator *pGenX,
                                                    RandomGenerator *pGenY );
    virtual   ~ShuffledMRandGen();

    virtual DIFFRET(RandomGenerator,ShuffledMRandGen)  *ReCreate( void ) const;

    virtual BOOL        bad( void ) const { return myBadFlag; }

    virtual RandGenToken  CreateToken( void ) const;
    virtual int     SetToken( const RandGenToken &Token );

    virtual  void        Reset( void );

    virtual  size_rand   Number( void );
    virtual  size_rand   MaxRandom( void ) const
     {
      return pRandGenX()->MaxRandom();
     }
   } ;

#endif
