/* =========================================================================
 *                                 RANDTABL.H
 * -------------------------------------------------------------------------
 *  Interface file for any table-driven random number generator as an object.
 *  (Assumes that the table resides on disk.)
 *
 *  Version 0.9
 *
 *  Copyright 1995-1997 by G. Wade Johnson
 *   Use of this code is placed in the public domain as long as the
 *   copyright notice is retained.
 */

#if !defined( RANDTABL_H )
  #define RANDTABL_H

  #if !defined( ENVIRON_H )
    #include "environ.h"
  #endif
  #include _FSTREAM_HEADER
  #if !defined( RANDGEN_H )
    #include "randgen.h"
  #endif


  class  TableRandGen : public RandomGenerator
   {
    ifstream         myTable;
    char            *myFileName;
    streampos        myStart;
    BOOL             myBadFlag;

  private:
    // protect the = operator and copy constructor so they cannot be used.
    TableRandGen& operator=( TableRandGen const & );
    TableRandGen( TableRandGen const & );

  protected:
    ifstream        &Table( void )           { return myTable; }
    const char      *FileName( void ) const  { return myFileName; }
    streampos        Start( void )    const  { return myStart; }
    void          SetStart( streampos start )  { myStart = start; }
    void          MakeBad( void ) { myBadFlag = true; }
    void          ClearBad( void ) { myBadFlag = true; }

  public:
    TableRandGen( const char *szFileName, streampos start );
    virtual   ~TableRandGen()
     {
      myTable.close();
      delete[] myFileName;
      myFileName = 0;
     }
    virtual BOOL        bad( void ) const { return myBadFlag; }
 
    virtual RandGenToken  CreateToken( void ) const;
    virtual int     SetToken( const RandGenToken &Token );

    virtual  void        Reset( void );

    virtual  void        IntSeed( size_rand seed );
    virtual  size_rand   Number( void )=0;
   } ;




  class  TextTableRandGen : public TableRandGen
   {
  private:
    // protect the = operator and copy constructor so they cannot be used.
    TextTableRandGen& operator=( TextTableRandGen const & );
    TextTableRandGen( TextTableRandGen const & );

  protected:
    void             InitTable( void );

  public:
    TextTableRandGen( const char *szFileName, streampos start );
    virtual   ~TextTableRandGen() { }
 
    virtual  void        Reset( void );

    virtual  size_rand   Number( void )=0;
   } ;




  class  BinaryTableRandGen : public TableRandGen
   {
  private:
    // protect the = operator and copy constructor so they cannot be used.
    BinaryTableRandGen& operator=( BinaryTableRandGen const & );
    BinaryTableRandGen( BinaryTableRandGen const & );

  public:
    EXPLICIT BinaryTableRandGen( const char *szFileName, streampos start=0 )
     : TableRandGen( szFileName, start )
     { }

    virtual   ~BinaryTableRandGen() { }
 
    virtual  size_rand   Number( void );
   } ;


#endif
