/* ========================================================================
 *                             RINGBUFF.CPP
 * ------------------------------------------------------------------------
 *  Implementation for a ring buffer class with iterators.
 *
 *  Version 0.9
 *
 *  Copyright 1997 by G. Wade Johnson
 *   Use of this code is placed in the public domain as long as the
 *   copyright notice is retained.
 */

#include "environ.h"

#include _IOSTREAM_HEADER

#include "ringbuff.h"




RingBuffer::RingBuffer( const RingBuffer &rhs )
  : mySize(0), myArray(0), myCursor(0), myFirstPass(true)
 {
  *this = rhs;
 }



RingBuffer &RingBuffer::operator=( const RingBuffer &rhs )
 {
  if(&rhs != this)
   {
    delete[] myArray;
    myArray     = 0;
    mySize      = rhs.max_size();
    myCursor    = rhs.myCursor;
    myFirstPass = rhs.myFirstPass;

    if(mySize)
     {
      myArray = new unsigned long[mySize];

      memcpy( myArray, rhs.myArray, sizeof(unsigned long)*mySize );
     }
   }

  return *this;
 }



RingBuffer::Iterator RingBuffer::Iterator::operator++()
 {
  assert(myBuffer && (unsigned)-1 != myIndex);

  if(++myIndex >= myBuffer->max_size())
    myIndex = 0;

  if(myIndex == myBuffer->Cursor())   // allow increment to end.
   {
    myIndex = (unsigned)-1;
   }

  return *this;
 }



RingBuffer::Iterator RingBuffer::Iterator::operator++( int )
 {
  Iterator temp = *this;
  ++(*this);
  return temp;
 }


RingBuffer::Iterator RingBuffer::Iterator::operator--()
 {
  assert(myBuffer);

  if(myIndex == (unsigned)-1)
    myIndex = myBuffer->Cursor();

  if(myIndex == 0)
    myIndex = myBuffer->max_size()-1;
  else
    myIndex--;

  return *this;
 }



RingBuffer::Iterator RingBuffer::Iterator::operator--( int )
 {
  Iterator temp = *this;
  --(*this);
  return temp;
 }


RingBuffer::Iterator &RingBuffer::Iterator::operator=( const Iterator &rhs )
 {
  myBuffer = rhs.myBuffer;
  myIndex  = rhs.myIndex;
  return *this;
 }



RingBuffer::Iterator  RingBuffer::begin()
 {
  if(myFirstPass)
   {
    if(0 == Cursor())
      return Iterator( this, (unsigned)-1 );
    else
      return Iterator( this, 0 );
   }
  else
    return Iterator( this, Cursor() );
 }


void   RingBuffer::Print( ostream &os ) const
 {
  char  c = '(';
  unsigned  i;

  os << '(' << mySize << ',' << myCursor;

  for(i=0;i < mySize;i++)
   {
    os << c << myArray[i];
    c = ',';
   }

  os << "))";
 }

void   RingBuffer::Read( istream &is )
 {
  char  dummy;
  unsigned  i;

  is >> dummy >> i >> dummy >> myCursor;
  if(i != mySize)
   {
    delete[] myArray;
    mySize = i;
    myArray = new unsigned long[mySize];
   }

  for(i=0;i < mySize;i++)
   {
    is >> dummy >> myArray[i];
   }

  is >> dummy >> dummy;
 }



ostream &operator<<( ostream &os, const RingBuffer &r )
 {
  r.Print( os );
  return os;
 }

istream &operator>>( istream &is, RingBuffer &r )
 {
  r.Read( is );
  return is;
 }
