/* =========================================================================
 *                                 RNDDWYER.CPP
 * -------------------------------------------------------------------------
 *  Implementation file for any linear congruential pseudo-random number 
 *  generator as an object.
 *
 *  This generator is based on the algorithm described in Jerry Dwyer's
 *  article in CUJ June 1995 entitled
 *     "Quick and Portable Random Number Generators".
 *
 *  WARNING: only valid for  Modulus > Multiplier * Multiplier.
 *
 *  Version 0.9
 *
 *  Copyright 1995-1997 by G. Wade Johnson
 *   Use of this code is placed in the public domain as long as the
 *   copyright notice is retained.
 */

#include "environ.h"
#include "rnddwyer.h"



DIFFRET(RandomGenerator,DwyerRandGen)  *DwyerRandGen::ReCreate( void ) const
 {
  return new DwyerRandGen( Multiplier(), Modulus(), Seed() );
 }




size_rand    DwyerRandGen::Number( void )
 {
  unsigned long  k = CurrSeed() / Quotient();
  long           residue;

  residue = Multiplier() * (CurrSeed() - k * Quotient()) - k * Remainder();

  while(residue < 0)
   {
    residue += Modulus();
   }

  SetCurrSeed( (unsigned long)residue % Modulus() );

  return CurrSeed();
 }
