/* =========================================================================
 *                                 RNDLCONM.H
 * -------------------------------------------------------------------------
 *  Interface file for any linear congruential pseudo-random number generator
 *  as an object.
 *
 *  Version 0.9
 *
 *  Copyright 1995-1997 by G. Wade Johnson
 *   Use of this code is placed in the public domain as long as the
 *   copyright notice is retained.
 */

#if !defined( RNDLCONM_H )
  #define RNDLCONM_H

  #if !defined( RANDGEN_H )
    #include "randgen.h"
  #endif
  #if !defined( RANDLCON_H )
    #include "randlcon.h"
  #endif


  class  LinConModRandGen : public LinConRandGen
   {
  private:
    const unsigned long   myModulus;

  private:
    // protect the = operator and copy constructor so they cannot be used.
    LinConModRandGen& operator=( LinConModRandGen const & );
    LinConModRandGen( LinConModRandGen const & );

  protected:
    unsigned long   Modulus( void ) const  { return myModulus; }

  public:
    LinConModRandGen( unsigned long mult, unsigned long incr,
                       unsigned long mod, size_rand seed=1 ) :
      LinConRandGen( mult, incr, seed ), myModulus(mod)
     {
      SetSeed( seed % Modulus() );
      LinConModRandGen::Reset();
     }
    virtual   ~LinConModRandGen() { }
 
    virtual DIFFRET(RandomGenerator,LinConModRandGen)  *ReCreate( void ) const;

    virtual  void        Reset( void ) { SetCurrSeed( Seed() ); }

    virtual  size_rand   Number( void );
    virtual  size_rand   MaxRandom( void ) const { return Modulus()-1; }
   } ;



#endif
